function hct = triangle(x,y,f,fx,fy,model)
%
% compute the three Clough-Tocher polynomial coefficients
%
d = [sqrt((x(3)-x(2))^2+(y(3)-y(2))^2),...
     sqrt((x(3)-x(1))^2+(y(3)-y(1))^2),...
     sqrt((x(2)-x(1))^2+(y(2)-y(1))^2)];
if (isa(f,'numeric'))
% f (and fx and fy) is a vector of length 3
  dof = [f(1),fx(1),fy(1),...
	 f(2),fx(2),fy(2),...
	 f(3),fx(3),fy(3),...
	 -(fx(1)+fx(2))/2*(y(2)-y(1))/d(3)+...
	 (fy(1)+fy(2))/2*(x(2)-x(1))/d(3),...
	 -(fx(2)+fx(3))/2*(y(3)-y(2))/d(1)+...
	 (fy(2)+fy(3))/2*(x(3)-x(2))/d(1),...
	 -(fx(1)+fx(3))/2*(y(1)-y(3))/d(2)+...
	 (fy(1)+fy(3))/2*(x(1)-x(3))/d(2)];
else
% f (and fx and fy) is a function
dof = [f(x(1),y(1)),fx(x(1),y(1)),fy(x(1),y(1)),...
       f(x(2),y(2)),fx(x(2),y(2)),fy(x(2),y(2)),...
       f(x(3),y(3)),fx(x(3),y(3)),fy(x(3),y(3)),...
       -fx((x(1)+x(2))/2,(y(1)+y(2))/2)*(y(2)-y(1))/d(3)+...
       fy((x(1)+x(2))/2,(y(1)+y(2))/2)*(x(2)-x(1))/d(3),...
       -fx((x(2)+x(3))/2,(y(2)+y(3))/2)*(y(3)-y(2))/d(1)+...
       fy((x(2)+x(3))/2,(y(2)+y(3))/2)*(x(3)-x(2))/d(1),...
       -fx((x(1)+x(3))/2,(y(1)+y(3))/2)*(y(1)-y(3))/d(2)+...
       fy((x(1)+x(3))/2,(y(1)+y(3))/2)*(x(1)-x(3))/d(2)];
end
if ((nargin == 5) || strcmp(model,'full'))
  a = HCT_sys(x(1),y(1),x(2),y(2),x(3),y(3),dof);
else
% reduced model
  a = red_HCT_sys(x(1),y(1),x(2),y(2),x(3),y(3),dof(1:9));
end
xbar = sum(x)/3;
ybar = sum(y)/3;
hct.x = [x(1),x(2),xbar;x(2),x(3),xbar;x(3),x(1),xbar];
hct.y = [y(1),y(2),ybar;y(2),y(3),ybar;y(3),y(1),ybar];
hct.coefs = [a(1:10);a(11:20);a(21:30)];
